local Lib = MissionMinder;
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local options = Lib.options;
local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local DEBUG = Lib.debug;

Lib.tracker = {};
local module = Lib.tracker;

local CHARACTERS = L["Characters"]
local COMPLETED = L["completed"]


local SECONDS = 1
local MINUTES = 60 * SECONDS
local HOURS = 60 * MINUTES
local DAYS = 24 * HOURS

local TIMES = { DAYS, HOURS, MINUTES}
local TIMESTRS = {INT_SPELL_DURATION_DAYS, INT_SPELL_DURATION_HOURS, INT_SPELL_DURATION_MIN }

local function secondsToString(seconds)
	local str = ""

	if  ( seconds < MINUTES ) then
		str = str .. string.format(INT_SPELL_DURATION_SEC, seconds);
	else
		local reportOn=2
		for i=1, #TIMES do
			if ( seconds >= TIMES[i] ) then
				if(reportOn <= 0) then
					break;
				end

				if ( str:len() > 0 ) then
					str = str .. " ";
				end

				str = str .. string.format(TIMESTRS[i], seconds / TIMES[i])
				seconds = seconds % TIMES[i]
				reportOn = reportOn - 1
			end
		end
	end

	return str
end

local function getPlayerName()
	return GetUnitName("player") .. "-" .. GetRealmName()
end

local function trackMission(event, missionID, ...)
	local player = getPlayerName()
	local now = GetTime()

	local inProgressMissions = MissionMinderGlobals.InProgressMissions
	if ( not inProgressMissions[player]) then
		inProgressMissions[player] = {}
	end

	local totalTimeInSeconds = select(2, C_Garrison.GetPartyMissionInfo(missionID));
	if (totalTimeInSeconds) then
		inProgressMissions[player][missionID] = now + totalTimeInSeconds
	end
end

local function untrackMission(event, missionID)
	local player = getPlayerName()

	local inProgressMissions = MissionMinderGlobals.InProgressMissions
	if ( inProgressMissions[player]) then
		inProgressMissions[player][missionID] = nil
	end
end

local function orderTrackedMissions(track1, track2)
	if ( track1.completed ~= track2.completed ) then
		return track1.completed > track2.completed
	elseif ( track1.lowest ~= track2.lowest ) then
		return track1.lowest < track2.lowest
	else
		return strcmputf8i(track1.name, track2.name) < 0;
	end
end


local function showTrackMissionTooltip()
	local now = GetTime();
	local currentPlayer = getPlayerName()

	local currentPlayerTbl = nil

	local tracking = {}
	local inProgressMissions = MissionMinderGlobals.InProgressMissions
	for player, missions in pairs(inProgressMissions) do
		local lowest = nil;
		local completed = 0

		for missionID, endDate in pairs(missions) do
			if ( endDate <= now ) then
				lowest = 0;
				completed = completed + 1
			elseif ( not lowest or endDate - now < lowest ) then
				lowest = endDate - now;
			end
		end

		if ( lowest ) then
			local tbl = {}
			tbl.name = player
			tbl.lowest = lowest
			tbl.completed = completed

			if (player == currentPlayer ) then
				currentPlayerTbl = tbl
			else
				tinsert(tracking, tbl)
			end
		end
	end

	if ( currentPlayerTbl ) then
		if ( currentPlayerTbl.completed > 0 ) then
				GameTooltip:AddLine(currentPlayerTbl.completed .. " " .. COMPLETED, 0, 1, 0);
		else
				GameTooltip:AddLine(secondsToString(currentPlayerTbl.lowest), 0.7, 0.7, 0.7);
		end
	end

	if ( #tracking > 0 ) then
		GameTooltip:AddLine(" ");
		GameTooltip:AddLine(CHARACTERS);
		table.sort(tracking, orderTrackedMissions);
		for i = 1, #tracking do
			if ( tracking[i].completed > 0 ) then
				GameTooltip:AddDoubleLine(tracking[i].name, tracking[i].completed .. " " .. COMPLETED, 1, 1, 1, 0, 1, 0);
			else
				GameTooltip:AddDoubleLine(tracking[i].name, secondsToString(tracking[i].lowest), 1, 1, 1, 0.7, 0.7, 0.7);
			end
		end;
	end

	GameTooltip:Show()
end

local function initialize()
	DEBUG("   |cffffffffTracker|r initialized");

	-- Track mission progress across multiple characters
	Lib.RegisterCallback("tracker", "MISSION_STARTED", trackMission)
	Lib.RegisterCallback("tracker", "MISSION_COMPLETED", untrackMission)
	GarrisonLandingPageMinimapButton:HookScript("OnEnter", showTrackMissionTooltip)
end

local events = Lib.events
events:RegisterEvent("tracker", "ADDON_LOADED", initialize);
